.peek-nav-section {
    padding: 16px;
    display: flex;
    justify-content: space-between;
    align-items: center;
    border-bottom: 1px solid #f0f0f0;
}

.login-steps-header {
    padding: 24px 16px;
    margin-bottom: 24px;
}

.plugin-user {
    padding: 0 16px;
}

.btn-group {
    display: flex;
    align-items: center;

    [nz-button] {
        min-width: 88px;
        display: flex;
        align-items: center;
        justify-content: center;

        [nz-icon] {
            margin: 0 8px;
            font-size: 14px;
        }
    }
}

:host ::ng-deep {
    .ant-steps {
        max-width: 800px;
        margin: 0 auto;
    }
}
